Imports System.IO
Imports System.Reflection
Imports System.Collections.Generic

Public Class Form1
    ' The assembly's information.
    Private m_Assembly As [Assembly] = Nothing

    ' Type objects for the assembly's types.
    Private m_Types As List(Of Type) = Nothing

    ' MethodInfo objects for the selected type's methods.
    Private m_MethodInfos As List(Of MethodInfo) = Nothing

    ' Try to find the DiscoveryLib dll.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim file_info As New FileInfo(Application.StartupPath & _
            "..\..\..\..\DiscoveryLib\bin\Debug\DiscoveryLib.dll")

        txtPath.Text = file_info.FullName
    End Sub

    ' Let the user browse for an assembly.
    Private Sub btnBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnBrowse.Click
        If dlgSelectAssembly.ShowDialog(Me) = Windows.Forms.DialogResult.OK Then
            txtPath.Text = dlgSelectAssembly.FileName
        End If
    End Sub

    ' Get information about the assembly.
    Private Sub btnGetAssemblyInfo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGetAssemblyInfo.Click
        ' Clear results.
        lstTypes.Items.Clear()
        lstMethods.Items.Clear()
        btnRun.Enabled = False

        ' Get the assembly's information.
        m_Assembly = [Assembly].LoadFrom(txtPath.Text)

        ' List the available types.
        m_Types = New List(Of Type)
        For Each exported_type As Type In m_Assembly.GetTypes()
            ' Skip the My namespace.
            If Not exported_type.Namespace.EndsWith(".My") Then
                ' Only use classes with default constructors.
                If HasDefaultConstructor(exported_type) Then
                    m_Types.Add(exported_type)
                    lstTypes.Items.Add(exported_type.Name)
                End If
            End If
        Next exported_type
    End Sub

    ' Return True if the type has a default constructor.
    Private Function HasDefaultConstructor(ByVal exported_type As Type) As Boolean
        For Each constructor_info As ConstructorInfo In exported_type.GetConstructors()
            If constructor_info.GetParameters().Length = 0 Then Return True
        Next constructor_info
        Return False
    End Function

    ' List the methods provided by this class.
    Private Sub lstTypes_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstTypes.SelectedIndexChanged
        If lstTypes.SelectedIndex < 0 Then Exit Sub

        ' Get the class's type information.
        Dim class_type As Type = m_Types(lstTypes.SelectedIndex)

        ' List the class's methods.
        m_MethodInfos = New List(Of MethodInfo)
        lstMethods.Items.Clear()
        For Each method_info As MethodInfo In class_type.GetMethods()
            m_MethodInfos.Add(method_info)
            lstMethods.Items.Add(MethodSignature(method_info))
        Next method_info

        btnRun.Enabled = False
    End Sub

    ' Enable or disable the Run button.
    Private Sub lstMethods_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstMethods.SelectedIndexChanged
        btnRun.Enabled = (lstMethods.SelectedIndex >= 0)
    End Sub

    ' Return a method's signature in a VB format.
    Private Function MethodSignature(ByVal method_info As MethodInfo) As String
        Dim txt As String = ""
        If method_info.IsFamilyAndAssembly Then
            txt &= "Protected Friend "
        ElseIf method_info.IsFamily Then
            txt &= "Protected "
        ElseIf method_info.IsFamilyOrAssembly Then
            txt &= "Friend "
        ElseIf method_info.IsPrivate Then
            txt &= "Private "
        ElseIf method_info.IsPublic Then
            txt &= "Public "
        End If
        If method_info.IsFinal Then txt &= "NotOverridable "
        If method_info.IsHideBySig Then txt &= "Overrides "
        If method_info.IsAbstract Then txt &= "MustOverride "
        If method_info.IsStatic Then txt &= "Shared "
        If method_info.ReturnType.Name = "Void" Then
            txt &= "Sub "
        Else
            txt &= "Function "
        End If
        txt &= method_info.Name
        txt &= ParameterList(method_info.GetParameters())
        If method_info.ReturnType.Name <> "Void" Then
            txt &= " As " & method_info.ReturnType.Name
        End If

        Return txt
    End Function

    ' Return a string listing parameters from a collection.
    Private Function ParameterList(ByVal param_infos() As ParameterInfo) As String
        Dim params As String = ""
        For Each parameter_info As ParameterInfo In param_infos
            params &= ", " & GetParameterData(parameter_info)
        Next parameter_info
        If params.Length > 0 Then params = params.Substring(2)

        Return "(" & params & ")"
    End Function

    ' Return information about a parameter.
    Private Function GetParameterData(ByVal parameter_info As ParameterInfo) As String
        Dim txt As String = ""
        If parameter_info.IsOptional Then
            txt &= "Optional "
        End If
        If parameter_info.IsOut Then
            txt &= "ByRef "
        Else
            txt &= "ByVal "
        End If
        txt &= parameter_info.Name
        If parameter_info.ParameterType.IsArray Then
            txt &= "()"
            txt &= " As " & parameter_info.ParameterType.GetElementType.Name
        Else
            txt &= " As " & parameter_info.ParameterType.Name
        End If
        If parameter_info.IsOptional Then
            If parameter_info.ParameterType.Name = "String" Then
                txt &= " = """ & parameter_info.DefaultValue.ToString() & """"
            Else
                txt &= " = " & parameter_info.DefaultValue.ToString()
            End If
        End If

        Return txt
    End Function

    ' Run the selected method.
    Private Sub btnRun_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRun.Click
        Try
            ' Get the selected method's information.
            Dim method_info As MethodInfo = m_MethodInfos(lstMethods.SelectedIndex)

            ' Get parameter info.
            Dim param_infos() As ParameterInfo = method_info.GetParameters()
            Dim param_values() As Object = Nothing
            If param_infos.Length > 0 Then
                ' Get parameters for the method call.
                Dim param_txt As String = InputBox("Parameters", "Parameters")
                If param_txt.Length = 0 Then Exit Sub
                Dim param_txts() As String = param_txt.Split(","c)
                If param_txts.Length <> param_infos.Length Then
                    MessageBox.Show("This method requires " & _
                        param_infos.Length & " parameters", _
                        "Invalid Parameters", _
                        MessageBoxButtons.OK, _
                        MessageBoxIcon.Exclamation)
                    Exit Sub
                End If

                ' Convert the parameters into the right data types.
                ReDim param_values(param_infos.Length - 1)
                For i As Integer = 0 To param_infos.Length - 1
                    param_values(i) = Convert.ChangeType( _
                        param_txts(i), _
                        param_infos(i).ParameterType)
                Next i
            End If

            ' Get the class's type information.
            Dim class_type As Type = m_Types(lstTypes.SelectedIndex)

            ' Create an instance of the class.
            Dim class_object As Object
            class_object = class_type.InvokeMember(Nothing, _
                BindingFlags.CreateInstance, Nothing, Nothing, Nothing)

            ' Invoke the method.
            Dim result As Object = method_info.Invoke(class_object, param_values)

            ' Display the result.
            MessageBox.Show( _
                "Result:" & vbCrLf & result.ToString(), _
                "Done", _
                MessageBoxButtons.OK, _
                MessageBoxIcon.Information)
        Catch ex As Exception
            MessageBox.Show( _
                "Error invoking method." & vbCrLf & ex.Message, _
                "Error Invoking Method", _
                MessageBoxButtons.OK, _
                MessageBoxIcon.Exclamation)
        End Try
    End Sub
End Class
